function [meanCoh] = Coh_bandpower(sig1, sig2, bands, sampRate, wavetype, FAST, rect, plotter)
%Coh_bandpower() Calculates the mean coherence in all bins across defined
%   frequency ranges in the wavelet coherence generated from wcoherence().
%   Inputs:
%       sig1 (array, required) = the first signal waveform as an array
%       sig2 (array, required) = the second signal waveform as an array
%       bands (list, required) = band boundaries in the format
%           [f1 f2 f3 f4 f5]
%       sampRate (int) = the sampling frequency of the input signal.
%           Default = 5000.
%       wavetype (int) = defines the mother wavelet as either 1 (amor) or
%           2 (bump). Default = Amor
%       FAST (int) = perform filtering on each band and recompose (1 = yes,
%           0 = no). Default = 1
%       rect (int) = rectify the signal? 1 = yes, 0 = no. Default = 0.
%       plotter (int) = plot the coherence? 1 = yes, 0 = no. Default = 0.
%
%   Output:
%       meanPower (cell) = the output of mean power in each band in each
%       row, with a column for each time point.
%
%	Use this format to use combination of default and specified arguments:
%	[meanCoh] = Coh_bandpower(inputsignal1, inputsignal2, 1:10:150, [], [] ,[], 0, 1)

% check inputs and set default arguments
if nargin < 4 || isempty(sampRate)
    sampRate = 5000; %default use 5000 Hz
end
if nargin < 5|| isempty(wavetype)
    wavetype = 1; %default use Amor
end
if nargin < 6 || isempty(FAST)
    FAST = 1; %default use FAST
end
if nargin < 7 || isempty(rect)
    rect = 0; %default use unrectified.
end
if nargin < 8 || isempty(plotter)
    plotter = 0; %default dont plot at the end.
end

% Select the options for wavetype: amor = 1, bump =2
wavelets = {'amor', 'bump'};
wave = wavelets{wavetype};

% Transpose the signals if required.
if size(sig1,1)>1
    sig1 = sig1';
    sig2 = sig2';
end

% Choose how to remove DC component
DCmethod = 0; % How to remove DC. 0 = whole signal, 1 = 20 ms window, 2 = 50 ms window

if DCmethod == 0
    % remove the DC component of the signal.
    DCcomp1 = mean(sig1);
    DCcomp2 = mean(sig2);
    sig1 = sig1-DCcomp1;
    sig2 = sig2-DCcomp2;
else
    % remove the DC component of the signal WINDOWED METHOD 20 ms
    if DCmethod == 1
        DCwindow = 0.02*sampRate;
        % remove the DC component of the signal WINDOWED METHOD 50 ms
    elseif DCmethod == 2
        DCwindow = 0.05*sampRate;
    end
    DCcomp1 = movmean(sig1,DCwindow);
    DCcomp2 = movmean(sig2,DCwindow);
    sig1 = sig1-DCcomp1;
    sig2 = sig2-DCcomp2;
end

% Zero-pad the signal prior to filtering.
sig_len = length(sig1);
sig1 = [zeros(1,sig_len), sig1, zeros(1,sig_len)];
sig2 = [zeros(1,sig_len), sig2, zeros(1,sig_len)];

% Apply same filters used at time of collection (10-1000 Hz)
collect_filt = 0;

if collect_filt >0
    collection_filter = designfilt('bandpassiir', ...
        'FilterOrder',4, ...
        'DesignMethod','butter', ...
        'HalfPowerFrequency1',1, ...
        'HalfPowerFrequency2',1000, ...
        'SampleRate',sampRate);
    sig1 = filtfilt(collection_filter, sig1);
    sig2 = filtfilt(collection_filter, sig2);
    % fvtool(collection_filter ) % uncomment to view filter
end

% Transpose the band array if required
if size(bands,1)>1
    bands = bands';
end

% specify cell for output
meanCoh = cell(size(bands,2)-1,1);

% loop through each frequency band
for i = 1:size(bands,2)-1
    
    % set the band boundaries and band edge width.
    f1 = bands(1,i);
    f2 = bands(1,i+1);
    bandedge = 1.5;
    
    % band edges
    if f1 - bandedge  < 1
        filband1 = 1;
    else
        filband1 = f1 - bandedge;
    end
    filband2 = f2 + bandedge;
    
    % If FAST, design and add filters
    if FAST > 0
        fil_band = designfilt('bandpassiir', ...
            'FilterOrder',4, ...
            'DesignMethod','butter', ...
            'HalfPowerFrequency1',filband1, ...
            'HalfPowerFrequency2',filband2, ...
            'SampleRate',sampRate);
        signal_fil1 = filtfilt(fil_band, sig1);
        signal_fil2 = filtfilt(fil_band, sig2);
        % fvtool(fil_band) % uncomment to view filter
        
    else
        signal_fil1 = sig1;
        signal_fil2 = sig2;
    end
    
    % Remove zero-padding
    signal_fil1 = signal_fil1(sig_len+1:(sig_len*2));
    signal_fil2 = signal_fil2(sig_len+1:(sig_len*2));
    
    % If rectification required, rectify signal
    if rect > 0
        signal_fil1 = abs(signal_fil1);
        signal_fil2 = abs(signal_fil2);
    end
    
    % Perform coherence
    if i == 1 | FAST > 0
        [wcoh, wcs, f] = wcoherence(signal_fil1, signal_fil2, sampRate, 'VoicesPerOctave', 32);
    end
    
    % Find all the bins in the wavelet coherence that fall within the band limits.
    freqbins = find(f >=  f1 & f <=  f2);
    
    % If no bins in this frequency range, mark this band
    % as -1. Otherwise, take the relevant rows from the coherence.
    if length(freqbins) < 1
        meanCoh{i}(1:length(wcoh)) = -1;
    else
        wcoh_out = wcoh(freqbins,:);
        % If there is only 1 bin in the range for this band, take the
        % absolute value of it, else take the mean of the absolute.
        if size(wcoh_out,1) > 1
            meanCoh{i} = mean(wcoh_out);
        else
            meanCoh{i} = wcoh_out;
        end
    end
    
    % To inspect output
    out_array = cell2mat(meanCoh);
end

% Plot the signal if specified
if plotter > 0
    xvalues = (1:sig_len)/sampRate;
    figure;plot(xvalues, sig1(sig_len+1:(sig_len*2)));
    figure;plot(xvalues, sig2(sig_len+1:(sig_len*2)));
    plot_spectrogram(meanCoh, bands, [0 1], sampRate);
end
end