function [event] = Coh_findlat(sig1, sig2, bands, samplingRate, loc, e, wave, FAST, rect)
%Coh_findlat() Calculates the coherence between two signals at specific
%   time point in defined frequency ranges using wcoher() function.
%   Inputs:
%       sig_1 (array, required) = the first signal waveform as an array
%       sig_2 (array, required) = the second signal waveform as an array
%       bands (list, required) = band boundaries in the format
%           [f1 f2 f3 f4 f5]
%       samplingRate (int) = the sampling frequency of the input signal.
%           Default = 5000.
%       loc (int) = the bin in the signal which corresponds to the stimulation
%           time
%       e (cell) = a list of bins in the signal from which to assess
%           frequency
%       wave (int) = defines the mother wavelet as either 1 (amor) or
%           2 (bump). Default = Amor
%       FAST (int) = perform filtering on each band and recompose (1 = yes,
%           0 = no). Default = 1
%       rect (int) = rectify the signal? 1 = yes, 0 = no. Default = 0.
%
%   Output:
%       event (cell) = the power in each band at each time point. Each time
%       point is corresponds to a different index in the cell, whereby the
%       first and second indices are the time in bins then s, followed by
%       an index for each band.

% check inputs and set default arguments
if nargin < 4 || isempty(samplingRate)
    samplingRate = 5000; %default use 5000 Hz
end
if nargin < 7 || isempty(wave)
    wave = 1; %default use Amor
end
if nargin < 8 || isempty(FAST)
    FAST = 1; %default use FAST
end
if nargin < 9 || isempty(rect)
    rect = 0; %default use unrectified.
end

%  convert cell to array and check how many events there are to measure.
f = cell2mat(e);
numeventtypes = length(f(~isnan(f)));

% run wcoher() on the signal
% Coh_bandpower(sig1, sig2, bands, sampRate, wavetype, FAST, rect, plotter)
bandcoherence = Coh_bandpower(sig1,sig2, bands, samplingRate, wave, FAST, rect);
bandsize = size(bandcoherence,1);

% variable for storing coherence at event timings
Ecoh = cell(numeventtypes,1);
for j = 1:numeventtypes
    Ecoh{j} = zeros(bandsize,1);
end

%  unpack the results and extract coherence
for i= 1:bandsize
    
    erange = cell(numeventtypes,1);
    for j = 1: numeventtypes
        erange{j} = bandcoherence{i}(1,e{j});
    end
    
    elat = cell(numeventtypes,1);
    ecoh = cell(numeventtypes,1);
    
    for k = 1:numeventtypes
        elat{k} = (e{k}-loc)*0.0002;
        ecoh{k} = erange{k};
        Ecoh{k}(i,1) = ecoh{k};
    end
    
    event = cell(numeventtypes,1);
    for l = 1:numeventtypes
        event{l} = cat(1,e{l}, elat{l},Ecoh{l});
    end
end
end