function [ done ] = WSST_Processall(inputfile, bands, sampfreq, wave, FAST, rect, foldername)
%WSST_Processall() extracts power in defined bands at time points stored in
%   the input files using Wavelet Sycnhrosqueezed Transform (WSST).
%   Parallel processing is used to speed up the operation.
% 
%   Inputs:
%       inputfile (int) = the input file number (usually stored one for
%           each subject).
%       bands (list, required) = band boundaries in the format
%           [f1 f2 f3 f4 f5]
%       sampfreq (int) = the sampling frequency of the input signal.
%           Default = 5000.
%       wave (int) = defines the mother wavelet as either 1 (amor) or
%           2 (bump). Default = Amor
%       FAST (int) = perform filtering on each band and recompose (1 = yes,
%           0 = no). Default = 1
%       rect (int) = rectify the signal? 1 = yes, 0 = no. Default = 0.
%       foldername (string) = where the data is saved. If empty, one is
%           generated automatically.
%
%   Output:
%       done (int) = indicates when operations have completed. Output is
%       saved inthe "Results" folder in pwd.

% check inputs and set default arguments
if nargin < 3 || isempty(sampfreq)
    sampfreq = 5000; %default use 5000 Hz
end
if nargin < 4 || isempty(wave)
    wave = 1; %default use Amor
end
if nargin < 5 || isempty(FAST)
    FAST = 1; %default use FAST
end
if nargin < 6 || isempty(rect)
    rect = 0; %default use unrectified.
end
if nargin < 7 || isempty(foldername)
    foldername = ''; %default generate folder name.
end

%% LOAD THE DATA
load('Workspace.mat');
readfile = inputfiles{1,inputfile};
present = pwd;
filedir = ('/Inputfiles/');
fullfiledir = strcat(present, filedir, readfile);
fileID = importdata(fullfiledir);
wavelets = {'amor', 'bump'};

% newtemp = strcat(pwd, '/TEMP/');
% 
% if exist(newtemp) == 0;
%     mkdir(newtemp);
% else
% end
% 
% setenv('TEMP',newtemp) % TEMP for Windows

setenv TZ UTC
getenv('TZ')

%% IMPORT THE DATA INTO VARIABLES
subsize = size(fileID.data,1);
titles = fileID.textdata(1,1:size(fileID.textdata,2));
eventtits = fileID.textdata(1,5:size(fileID.textdata,2));
subarray =  fileID.textdata(2:(subsize+1),1);
segs = fileID.textdata(2:size(fileID.textdata,1),2);
conditions = fileID.textdata(2:size(fileID.textdata,1),3);
muscles = fileID.textdata(2:size(fileID.textdata,1),4);

%% COUNT NUMBER OF RESULTS FROM INPUT FILE AND MAKE A TABLE THE SAME SIZE
% cell array for events. one cell per input.
earray = cell(subsize,1);
% Greatest number of event types in input file
maxnoeventtypes = size(fileID.data,2);

% fill one cell for each sub with all events for that input
for o = 1:subsize
    earray{o} = fileID.data(o,:);
end

%% MAKE TABLE AND HEADINGS FOR RESULTS

%% MAKE HEADERS
Coltits = cell(1);
parfor i = 1:size(bands,2)-1
    f1 = bands(1,i);
    %f2 = bands(1,i+1);
    %    tits = [num2str(f1) ' - ' num2str(f2) ' Hz'];          Uncomment this
    %    line + comment next line for " x - y Hz" format for band title.
    tits = num2str(f1); % leave this line in for "x" format for band title.
    Coltits{1,i} = tits;
end
Heads = {'Event', 'Matlab Input','Latency'};
Colheads = cat(2,titles(1,1:4),Heads, Coltits);

%% WORK OUT HOW BIG THE CELL FOR THE RESULTS FILE NEEDS TO BE
numsubmissions = 0;
numres = 0;
for i = 1:length(subarray)
    segchar = char(segs(i));
    segvec = sscanf(segchar,'%f');
    numsegs = length(segvec);
    numevents = length(earray{i}(~isnan(earray{i})));
    if strfind(conditions{i},'cont_iso') > 0
        numressub = (numsegs * numevents) * 2;
        numsubmissionsseg = numsegs * 2;
    else
        numressub = (numsegs * numevents);
        numsubmissionsseg = numsegs;
    end
    numres = numres + numressub;
    numsubmissions = numsubmissions + numsubmissionsseg;
    
end

numheads = (size(fileID.textdata,2) + size(bands,2)) - 1;
allresults = cell(numres+1,(numheads-2));
allresults(1,1:size(Colheads,2)) = Colheads(:,1:size(Colheads,2));

dataforprc = cell(numsubmissions,1);

%% FILL CATEGORICAL DATA
x = 1;
emptyrow = 2;
for j = 1:length(subarray)
    sub = subarray(j);
    condition = char(conditions(j));
    muscle = char(muscles(j));
    segchar = char(segs(j));
    segvec = sscanf(segchar,'%f');
    numsegs = size(segvec,1);
    numeventsseg = length(earray{j}(~isnan(earray{j})));
    
    %Check there is segs defined
    if isempty(segvec) == 1
        continue
    else
    end
    
    %Check there is events defined
    if isempty(earray{j}) == 1
        continue
    else
    end
    
    cont = contains(condition,'cont'); %0
    
    if cont > 0
        loc = 0;
        segcond = erase(condition, 'cont_');
        if contains(condition, 'cycle') > 0
            eventtype = repmat({'cycle'},1,numeventsseg);
        else contains(condition, 'iso') > 0;
            eventtype = repmat({'iso'},1,numeventsseg);
        end
    else  
        if contains(condition, 'mep_cycle') > 0  && contains(condition, 'cmep') < 1 %5054
            loc = 5054;
        elseif contains(condition, 'cmep_cycle') > 0 %5002
            loc = 5002;
        elseif contains(condition, 'mwave_cycle') > 0 %5002
            loc = 5002;
       elseif contains(condition, 'mwave_iso') > 0 %10000
            loc = 10000;
        elseif contains(condition, 'mep_iso') >0 && contains(condition, 'cmep') < 1  %10055
            loc = 10055;
        elseif contains(condition, 'cmep_iso') %10040
            loc = 10040;
        end
        
        eventtype = eventtits';
        
        segcond = condition;
    end
    
    rescond = condition;
    
    e = cell(numeventsseg,1);
    for p = 1:numeventsseg
        e{p} = round(loc+(earray{j}(1,p)/0.0002));
    end
    
    for l = 1:numsegs
        seg = segvec(l);
        segstr = sprintf('%d',seg);
        inputsigstr = strcat(muscle,'_',sub,'_', segcond,'_','seg{',segstr,'}');
        inputsigchar=char(inputsigstr);
        sig=eval(inputsigchar);
        
        printevents = ['File = ',inputsigchar,', File size = ', num2str(size(sig,1)), ', Loc = ', num2str(loc), ', Events = ',num2str([e{1:numeventsseg}])]
        printevents = ['Segcond = ',segcond, ' Rescond = ',rescond]
        
        
        %V IMPORTANT DONT REMOVE
        dataforprc{x} = {sig,bands,sampfreq,loc,e,maxnoeventtypes,wave}';
        x = x + 1;
        %V IMPORTANT DONT REMOVE
        
        for k = 1: numeventsseg
            segstr = segvec(l);
            allresults{emptyrow,1} = sub{1};
            allresults{emptyrow,2} = segstr;
            allresults{emptyrow,3} = rescond;
            allresults{emptyrow,4} = muscle;
            allresults{emptyrow,5} = eventtype{k};
            emptyrow = emptyrow + 1;
        end
    end
end

FinalResults = parproc(dataforprc, allresults, FAST, rect);

banddif = bands(2)-bands(1);
bandchar = sprintf('%d', banddif);
folder1 = strcat('Results');
if foldername == ''
    folder2 = strcat(wavelets{wave}, '_bandssize_', bandchar);
else
    folder2 = foldername;
end
resultssubfolderdir = fullfile(present, folder1, folder2);
resultfulldir = fullfile(present, folder1, folder2, readfile);

if exist(resultssubfolderdir) == 0
    mkdir(resultssubfolderdir);
else
end

cell2csv(resultfulldir, FinalResults, ';', 2013, '.');
done = 1;
end

%% Do analysis in parallel
function [FinalResults] = parproc(alldata, allresults, FAST, rect)
%% Step 1: create a mat-file per worker using SPMD
spmd
    myFname = tempname(); % each worker gets a unique filename
    myMatfile = matfile(myFname, 'Writable', true);
end
%% Step 2: create a parallel.pool.Constant from the 'Composite'
% This allows the worker-local variable to used inside PARFOR
myMatfileConstant = parallel.pool.Constant(myMatfile);

%% Step 3: run PARFOR
parfor idx = 1:length(alldata)
    data = alldata{idx};
    % 'data' is stored in the format:
    %   dataforprc{x} = {sig,bands,sampfreq,loc,e,maxnoeventtypes,wave};
	%	where:
    % data{1} = signal
    % data{2} = bands
    % data{3} = sampling frequency
    % data{4} = loc
    % data{5} = events
    % data{6} = max number of event types (not input into WSST)
    % data{7} = wavetype
    % these variables feed into the WSST_findlat function in this order: 
	%[event] WSST_findlat(sig, bands, samplingRate, loc, e, wave, FAST, rect)
    [event] = WSST_findlat(data{1},data{2},data{3},data{4},data{5},data{7}, FAST, rect);
    evlen = length(event);
    matfileObj = myMatfileConstant.Value;
    % Append into 'testOut', storing the index
    matfileObj.testOut(1:evlen,idx) = event;
    matfileObj.gotResult(1:evlen,idx) = true;
    %parout = parallel(alldata,idx,myMatfileConstant)
end

%% Step 4: accumulate the results on the client
% Here we retrieve the filenames from 'myFname' Composite,
% and use them to accumulate the overall result
outmatfile = struct();
outmatfile.out = cell(alldata{1}{6},length(alldata));
save out.mat outmatfile -v7.3

% size of workerMatfile.testOut(1, jdx);
for idx = 1:numel(myFname)
    workerFname = myFname{idx};
    workerMatfile = matfile(workerFname);
    workerOutSz = size(workerMatfile, 'testOut');
    for jdx = 1:workerOutSz(2)
        if workerMatfile.gotResult(1,jdx)
            for kdx = 1:workerOutSz(1)
                outmatfile.out(kdx,jdx) = workerMatfile.testOut(kdx,jdx);
            end
        end
    end
end


alresind = 2;
for i = 1:size(outmatfile.out,2)
    {outmatfile.out{1,i}};
    %Check number of events
    for evchk = 1:size(outmatfile.out,1)
        testcellfilled = isempty(outmatfile.out{evchk,i});
        if  testcellfilled == 1
            evlength = evchk -1;
            break
        else
            testcellfilled = 0;
            evlength = evchk;
        end
    end
    
    for j = 1:evlength
        for k = 1:size(outmatfile.out{j,i},1)
            allresults(alresind,k+5) = {outmatfile.out{j,i}(k)};
        end
        alresind = alresind +1;
    end
end

FinalResults = allresults;
done = 1;
end