function [meanPower] = WSST_bandpower(signal, bands, sampRate, wavetype, FAST, rect, plotter)
%WSST_bandpower() Calculates the mean power in all bins across defined
%   frequency ranges in the WSST.
%   Inputs:
%       signal (array, required) = the signal waveform as an array
%       bands (list, required) = band boundaries in the format
%           [f1 f2 f3 f4 f5]
%       sampRate (int) = the sampling frequency of the input signal.
%           Default = 5000.
%       wavetype (int) = defines the mother wavelet as either 1 (amor) or
%           2 (bump). Default = Amor
%       FAST (int) = perform filtering on each band and recompose (1 = yes,
%           0 = no). Default = 1
%       rect (int) = rectify the signal? 1 = yes, 0 = no. Default = 0.
%       plotter (int) = plot the WSST? 1 = yes, 0 = no. Default = 0.
%
%   Output:
%       meanPower (cell) = the output of mean power in each band in each
%       row, with a column for each time point.
%
%	Use this format to use combination of default and specified arguments:
%	[wsst_out] = WSST_bandpower(inputsignal, 1:10:150, [], [] ,[], 0, 1)

% check inputs and set default arguments
if nargin < 3 || isempty(sampRate)
    sampRate = 5000; %default use 5000 Hz
end
if nargin < 4|| isempty(wavetype)
    wavetype = 1; %default use Amor
end
if nargin < 5 || isempty(FAST)
    FAST = 1; %default use FAST
end
if nargin < 6 || isempty(rect)
    rect = 0; %default use unrectified.
end
if nargin < 7 || isempty(plotter)
    plotter = 0; %default dont plot wsst at the end.
end

% Select the options for wavetype: amor = 1, bump =2
wavelets = {'amor', 'bump'};
wave = wavelets{wavetype};

% Transpose the signal if required.
if size(signal,1)>1
    signal = signal';
end

% Choose how to remove DC component
DCmethod = 0; % How to remove DC. 0 = whole signal, 1 = 20 ms window, 2 = 50 ms window

if DCmethod == 0
    % remove the DC component of the signal.
    DCcomp = mean(signal);
    signal = signal-DCcomp;
else
    % remove the DC component of the signal WINDOWED METHOD 20 ms
    if DCmethod == 1
        DCwindow = 0.02*sampRate;
        % remove the DC component of the signal WINDOWED METHOD 50 ms
    elseif DCmethod == 2
        DCwindow = 0.05*sampRate;
    end
    DCcomp = movmean(signal,DCwindow);
    signal = signal-DCcomp;
end

% Zero-pad the signal prior to filtering.
sig_len = length(signal);
signal = [zeros(1,sig_len), signal, zeros(1,sig_len)];

% Apply same filters used at time of collection (10-1000 Hz)
collect_filt = 0;

if collect_filt >0
    collection_filter = designfilt('bandpassiir', ...
        'FilterOrder',4, ...
        'DesignMethod','butter', ...
        'HalfPowerFrequency1',10, ...
        'HalfPowerFrequency2',1000, ...
        'SampleRate',sampRate);
    signal = filtfilt(collection_filter, signal);
    % fvtool(collection_filter ) % uncomment to view filter
end

% Transpose the band array if required
if size(bands,1)>1
    bands = bands';
end

% specify cell for output
meanPower = cell(size(bands,2)-1,1);

% loop through each frequency band
for i = 1:size(bands,2)-1
    
    % set the band boundaries and band edge width.
    f1 = bands(1,i);
    f2 = bands(1,i+1);
    bandedge = 1.5;
    
    % band edges
    if f1 - bandedge  < 1
        filband1 = 1;
    else
        filband1 = f1 - bandedge;
    end
    filband2 = f2 + bandedge;
    
    % If FAST, design and add filters
    if FAST > 0
        fil_band = designfilt('bandpassiir', ...
            'FilterOrder',4, ...
            'DesignMethod','butter', ...
            'HalfPowerFrequency1',filband1, ...
            'HalfPowerFrequency2',filband2, ...
            'SampleRate',sampRate);
        signal_fil = filtfilt(fil_band, signal);
        % fvtool(fil_band) % uncomment to view filter
        
    else
        signal_fil = signal;
    end
    
    % Remove zero-padding
    signal_fil = signal_fil(sig_len+1:(sig_len*2));
    
    % If rectification required, rectify signal
    if rect > 0
        signal_fil = abs(signal_fil);
    end
    
    % Perform WSST
    if i == 1 || FAST > 0
        [sst, f] = wsst(signal_fil, sampRate, wave, 'VoicesPerOctave', 48);
    end
    
    % Find all the bins in the WSST that fall within the band limits.
    freqbins = find(f >=  f1 & f <=  f2);
    
    % If no bins in this frequency range, mark this band
    % as -1. Otherwise, take the relevant rows from the wsst.
    if length(freqbins) < 1
        meanPower{i}(1:length(sst)) = -1;
    else
        sst_out = sst(freqbins,:);
        % If there is only 1 bin in the range for this band, take the
        % absolute value of it, else take the mean of the absolute.
        if size(sst_out,1) > 1
            meanPower{i} = mean(abs(sst_out));
        else
            meanPower{i} = abs(sst_out);
        end
    end
    
    % To inspect output
    out_array = cell2mat(meanPower);
end

% Plot the signal if specified
if plotter > 0
    xvalues = (1:sig_len)/sampRate;
    figure;plot(xvalues, signal(sig_len+1:(sig_len*2)));
    plot_spectrogram(meanPower, bands, [], sampRate);
end
end