function [event] = WSST_findlat(sig, bands, samplingRate, loc, e, wave, FAST, rect)
%WSST_findlat() Calculates the power at specific time point in defined
%   frequency ranges in the WSST.
%   Inputs:
%       sig (array, required) = the signal waveform as an array
%       bands (list, required) = band boundaries in the format
%           [f1 f2 f3 f4 f5]
%       samplingRate (int) = the sampling frequency of the input signal.
%           Default = 5000.
%       loc (int) = the bin in the signal which corresponds to the stimulation
%           time
%       e (cell) = a list of bins in the signal from which to assess
%           frequency
%       wave (int) = defines the mother wavelet as either 1 (amor) or
%           2 (bump). Default = Amor
%       FAST (int) = perform filtering on each band and recompose (1 = yes,
%           0 = no). Default = 1
%       rect (int) = rectify the signal? 1 = yes, 0 = no. Default = 0.
%
%   Output:
%       event (cell) = the power in each band at each time point. Each time
%       point is corresponds to a different index in the cell, whereby the
%       first and second indices are the time in bins then s, followed by
%       an index for each band.

% check inputs and set default arguments
if nargin < 3 || isempty(samplingRate)
    samplingRate = 5000; %default use 5000 Hz
end
if nargin < 6 || isempty(wave)
    wave = 1; %default use Amor
end
if nargin < 7 || isempty(FAST)
    FAST = 1; %default use FAST
end
if nargin < 8 || isempty(rect)
    rect = 0; %default use unrectified.
end

%  convert cell to array and check how many events there are to measure.
f = cell2mat(e);
numeventtypes = length(f(~isnan(f)));

% run the WSST on the signal
% WSST_bandpower(signal, bands, sampRate, wavetype, FAST, rect, plotter)
bandPowers = WSST_bandpower(sig, bands, samplingRate, wave, FAST, rect);
bandsize = size(bandPowers,1);

% variable for storing power at event timings
Epowers = cell(numeventtypes,1);
for j = 1:numeventtypes
    Epowers{j} = zeros(bandsize,1);
end

%  unpack the results and extract power
for i= 1:bandsize
    
    erange = cell(numeventtypes,1);
    for j = 1: numeventtypes
        erange{j} = bandPowers{i}(1,e{j});
    end
    
    elat = cell(numeventtypes,1);
    epower = cell(numeventtypes,1);
    
    for k = 1:numeventtypes
        elat{k} = (e{k}-loc)*0.0002;
        epower{k} = erange{k};
        Epowers{k}(i,1) = epower{k};
    end
    
    event = cell(numeventtypes,1);
    for l = 1:numeventtypes
        event{l} = cat(1,e{l}, elat{l},Epowers{l});
    end
end
end