function [done] = run(subjectno, bandsno, UID, todays_date, analysis, sampFreq, wave, FAST, rect)
%run() organizes WSST_Processall and Coh_Processall to perform the analysis 
%using specific input parameters. A folder is generated from the UID input argument 
%the results are saved there. Saves a .csv file which details input parameters in the 
%same folder as the results.
%
%   Inputs:
%       subjectno (int) = the number of the subject from 1-10 in the order
%           aa, am, bh, cg, jd, jg, mattp, mk, ps, rd.
%       bandsno (int) = band boundary configuration whereby:
%           1 = [1:2]; % test
%           2 = [1:151]; % 1 Hz bins
%           3 = [15 35 59 61 120 150]; % EEG bins
%       UID (str) = a UID generated and passed through the BASH script.
%       analysis = analysis type whereby 0 = WSST, 1 = wavelet coherence.
%           Default = 0, WSST.
%       sampfreq (int) = the sampling frequency of the input signal.
%           Default = 5000.
%       wave (int) = defines the mother wavelet as either 1 (amor) or
%           2 (bump). Default = Amor
%       FAST (int) = perform filtering on each band and recompose (1 = yes,
%           0 = no). Default = 1
%       rect (int) = rectify the signal? 1 = yes, 0 = no. Default = 0.

% convert UID to string
UID = string(UID);

% check date input
if nargin < 4 || isempty(todays_date)
    todays_date = datetime(date);
    todays_date.Format = 'yyyy-MM-dd';
else
    todays_date = string(todays_date);
end

% check inputs and set default arguments
if nargin < 5 || isempty(analysis)
    analysis = 0; %default use WSST
end
if nargin < 6 || isempty(sampFreq)
    sampFreq = 5000; %default use 5000 Hz
end
if nargin < 7 || isempty(wave)
    wave = 1; %default use Amor
end
if nargin < 8 || isempty(FAST)
    FAST = 1; %default perform FAST
end
if nargin < 9 || isempty(rect)
    rect = 0; %default use unrectified
end

% to configure for parallel processing
distcomp.feature( 'LocalUseMpiexec', false );

% define which bands to use
switch(bandsno)
    case 1
        bands = [1:2]; % test
        bandstr = 'BandsTest';
    case 2
        bands = [1:151]; % 1 Hz bins
        bandstr = 'Bands1Hz';
    case 3
        bands = [15 35 59 61 120 150]; % EEG bins
        bandstr = 'BandsEEG';
    otherwise
        fprintf('Invalid band input\n' );
end

% define analysis string
switch(analysis)
    case 0
        analysisstr = 'WSST';
    case 1
        analysisstr = 'WCoh';
    otherwise
        fprintf('Invalid analysis type input\n' );
end

% define wave string
switch(wave)
    case 1
        wavestr = 'Amor';
    case 2
        wavestr = 'Bump';
    otherwise
        fprintf('Invalid wavelet type input\n' );
end

% define rectification string
switch(rect)
    case 0
        rectstr = 'Unrect';
    case 1
        rectstr = 'Rect';
    otherwise
        fprintf('Invalid rectification input\n' );
end

% define filtering string
switch(FAST)
    case 0
        FASTstr = 'NoFilt';
    case 1
        FASTstr = 'FAST';
    otherwise
        fprintf('Invalid filtering input\n' );
end

% configure folder name and details to save in .csv file
results_dir_root = strcat(pwd, '/Results');
bandsinfo = join(string(bands));
UIDstr = strcat('UID-', UID);

% define output folder name and directory
foldername = strcat(analysisstr, '_', string(todays_date), '_', ...
    wavestr, '_', bandstr, '_', FASTstr, '_', rectstr, '_', UIDstr)
results_dir_sub = strcat(results_dir_root, '/', foldername);
% results_dir_sub = strcat('./Results/', foldername);

% make the directory and results folder if it doesnt exist
if not(isfolder(results_dir_sub))
    % make new results subdirectory
    mkdir(results_dir_sub);
    
    % make and save .csv configuation file
    cell_out = {'Analysis:', analysisstr;
        'Date:', string(todays_date); 
        'Unique ID:', UIDstr; 
        'Bands:', bandsinfo; 
        'Wavelet type:', wavestr; 
        'Rectified?', rectstr; 
        'Filtering?', FASTstr};
    writecell(cell_out, strcat(results_dir_sub, '.csv'));
end

% perform the work
if analysis == 0
    WSST_Processall(subjectno, bands, sampFreq, wave, FAST, rect, foldername);
else
    Coh_Processall(subjectno, bands, sampFreq, wave, FAST, rect, foldername);
end

% job is finished
done  = 1;

% quit matlab (requirement for HPC).
quit;
end