library(dplyr)
library(tidyr)

# Function for standard error
sem <- function(x) sd(x)/sqrt(length(x))

# WORK STARTS HERE -------------------------------------------------------------
# set working directory to the folder containing the script
root_dir <- dirname(rstudioapi::getActiveDocumentContext()$path)
print(root_dir)
setwd(root_dir)

# look for results folder 
resultsdir <- file.path(dirname(root_dir), 'Step2_SortForMatlab')

# check or create general outputdir
outputdir <- file.path(root_dir, 'TimeToPeak_stats')
if (file.exists(outputdir) == FALSE){
  dir.create(file.path(outputdir))
}


# subset initial data for event P1
all_data <- read.csv(file = paste0(resultsdir, "/Alldata.csv"), check.names = FALSE) %>%
  select(Subject, Condition, Muscle, P1) %>%
  filter(!grepl('cont*|mwave*', Condition)) %>%
  mutate(P1 = round(P1 * 1000, digits = 2),
         Subject = as.factor(Subject),
         Condition = as.factor(Condition),
         Muscle = as.factor(Muscle))

# for n numbers
trials_per_subject <- all_data %>% 
  group_by(Condition, Subject, Muscle)%>%
  summarise(N = n()) %>%
  ungroup() %>%
  pivot_wider(names_from = Subject, values_from = N) %>%
  group_by(Condition, Muscle) %>% 
  mutate(Total = sum(c_across(aa:rd)),
         across(where(is.numeric), replace_na, 0),
         Total_subjects = sum(c_across(aa:rd) != 0))%>% 
  arrange(Condition)

  # save as csv
write.csv(trials_per_subject, paste(outputdir,'/N_trials_per_sub.csv', sep=''))

#### DO ANOVAs
data_for_anova <- all_data %>%
  mutate(Condition = paste(Condition, Muscle, sep="_"))

anova_results <- aov(P1 ~ Subject + Condition, data = data_for_anova)
anova_dataframe <- as.data.frame(summary(anova_results)[[1]])

tukeyresults <- TukeyHSD(anova_results, ("Condition"), conf.level=.99,ordered = FALSE) 

tukeydf <- as.data.frame(tukeyresults$Condition)

#Add Bonferonni correction
tukeyP <- as.matrix(tukeydf[4])
tukeyPcorrected <- p.adjust(tukeyP, method = "bonferroni", n = length(tukeyP))
# https://stats.stackexchange.com/questions/9425/bonferroni-or-tukey-when-does-the-number-of-comparisons-become-large
tukeyPcorrecteddf <- tukeydf
tukeyPcorrecteddf[4] <- tukeyPcorrected

# write results to file
write.csv(tukeyPcorrecteddf, file.path(outputdir, "Anova_Two-way_TukeyPvals.csv"))
write.csv(anova_dataframe, file.path(outputdir, "Anova_Two-way.csv"))

# to subset for the desired comparisons
alltukeyresnames <- unique(rownames(tukeyPcorrecteddf))
option1 <- unique(gsub(".*-", "", rownames(tukeyPcorrecteddf)))
option2 <- unique(gsub("-.*", "", rownames(tukeyPcorrecteddf)))

desiredtest1 <- paste(option1, gsub("biceps", "triceps", option1), sep = "-")
desiredtest2 <- paste(option1, gsub("triceps", "biceps", option1), sep = "-")
desiredtest3 <- paste(option2, gsub("biceps", "triceps", option2), sep = "-")
desiredtest4 <- paste(option2, gsub("triceps", "biceps", option2), sep = "-")

allpossible <- c(desiredtest1, desiredtest2, desiredtest3, desiredtest4)

BicVsTric <- tukeyPcorrecteddf %>%
  tibble::rownames_to_column("test") %>%
  filter(test %in% allpossible)

write.csv(BicVsTric, paste0(outputdir, "/Anova_Two-way_TukeyPvals_Bic_Tric.csv"), quote = TRUE)

# calculate mean, mean of means, and sem for plot
Mean <- all_data %>%
  group_by(Subject, Condition, Muscle) %>%
  summarise(P1 = round(mean(P1), digits = 2)) %>%
  arrange(Condition)

Meanofmean <- Mean %>%
  group_by(Condition, Muscle) %>%
  summarise(Latency = round(mean(P1), digits = 2),
            SE = round(sem(P1), digits = 2)) %>%
  mutate(neger = Latency-SE,
         poser = Latency+SE) %>%
  arrange(Condition)

# Print ANOVA stats
anova_df <- anova_dataframe$Df[6]
anova_f <- round(anova_dataframe$'F value'[6], digits = 2)
anova_p <- anova_dataframe$'Pr(>F)'[6]
p_sig <- ifelse(anova_p < 0.01, "p < 0.01", "NS")

print(paste("Two-way ANOVA, F(", anova_df, ")=", anova_f,", ", p_sig, ", n = 10", sep=""))

for (condition in unique(Meanofmean$Condition)){
  cond_mean_sem <- Meanofmean %>% 
    filter(Condition == condition)
  
  bic_mean <- cond_mean_sem$Latency[cond_mean_sem$Muscle == "biceps"]
  bic_sem <- cond_mean_sem$SE[cond_mean_sem$Muscle == "biceps"]
  bic_n <- trials_per_subject$Total_subjects[trials_per_subject$Condition == condition &
                                               trials_per_subject$Muscle == "biceps"]
  tric_mean <- cond_mean_sem$Latency[cond_mean_sem$Muscle == "triceps"]
  tric_sem <- cond_mean_sem$SE[cond_mean_sem$Muscle == "triceps"]
  tric_n <- trials_per_subject$Total_subjects[trials_per_subject$Condition == condition &
                                               trials_per_subject$Muscle == "triceps"]
  
  expression <- paste(condition, ": biceps brachii: ", bic_mean, "  ", bic_sem, " ms, n = ", bic_n,
                      "; triceps brachii: ", tric_mean, "  ", tric_sem, " ms, n = ", tric_n, sep="")
  
  # check the stats
  cond_stats <- BicVsTric %>%
    filter(grepl(paste("-",condition,sep=""), test))
  
  
  sig <- ifelse(cond_stats$`p adj` < 0.01, "p < 0.01", "NS")

  print(paste(expression, "; ", sig, sep=""))
  
}

mep_cmep_dif <- Meanofmean %>%
  pivot_wider(id_cols = Muscle, names_from = Condition, values_from = Latency) %>%
  group_by(Muscle) %>%
  summarise(cycle_6 = mep_cycle_6 - cmep_cycle_6,
            cycle_12 = mep_cycle_12 - cmep_cycle_12,
            iso_6 = mep_iso_6 - cmep_iso_6,
            iso_12 = mep_iso_12 - cmep_iso_12)






Meanofmean_output <- Mean %>%
  pivot_wider(id_cols = c(Subject, Condition), names_from = Muscle, values_from = c(P1)) %>%
  group_by(Condition) %>%
  mutate(dif = biceps - triceps) %>%
  ungroup() %>%
  group_by(Condition) %>%
  summarise(Biceps = round(mean(biceps), digits = 2),
            Triceps = round(mean(triceps), digits = 2),
            Dif = round(mean(dif), digits = 2),
            Biceps_sem = round(sem(biceps), digits = 2),
            Triceps_sem = round(sem(triceps), digits = 2),
            Dif_sem = round(sem(dif), digits = 2)) %>%
  ungroup() %>%
  mutate(Order = order(c("mep_cycle_6",
                             "cmep_cycle_6",
                             "mep_cycle_12",
                             "cmep_cycle_12",
                             "mep_iso_6",
                             "cmep_iso_6",
                             "mep_iso_12",
                             "cmep_iso_12"))) %>%
  arrange(Order) %>%
  select(-Order)

write.csv(Meanofmean_output, paste0(outputdir, "/TimeToPeak_means_dif.csv"), quote = TRUE)