  # Setup ------------------------------------------------------------------------
library(dplyr)
library(tidyr)
library(ggplot2)
library(ggsignif)
library(ggpubr)
library(rstatix)

# Set up fonts for plotting.
# library(extrafont)
# library(showtext)
# font_add('Calibri', 'Calibri.ttf')
# showtext_auto()

# Function for standard error
sem <- function(x) sd(x)/sqrt(length(x))

# set working directory to the folder containing the script
root_dir <- dirname(rstudioapi::getActiveDocumentContext()$path)
print(root_dir)
setwd(root_dir)

# set up folders for output
fig_dir <- "Figures"
ttp_fig_dir <- "Figures/TimeToPeak"

if (file.exists(fig_dir) == FALSE){
  dir.create(file.path(root_dir, fig_dir))
}

if (file.exists(ttp_fig_dir) == FALSE){
  dir.create(file.path(root_dir, ttp_fig_dir))
}

# subset initial data for event P1
all_data <- read.csv(file = "../Step2_SortForMatlab/AllData.csv", 
                     check.names = FALSE) %>%
  select(Subject, Condition, Muscle, P1) %>%
  filter(!grepl('cont*|mwave*', Condition)) %>%
  mutate(P1 = round(P1 * 1000, digits = 2),
         Subject = as.factor(Subject),
         Condition = as.factor(Condition),
         Muscle = as.factor(Muscle)) %>%
  # sort mean of mean data and add labels for faceting
  mutate(Task = as.factor(ifelse(grepl('cycle', Condition), 'Cycle', 'Tonic')),
         Clock = as.factor(ifelse(grepl('6', Condition), "6 o'clock", "12 o'clock")),
         Stim = as.factor(ifelse(grepl('cmep', Condition), 'TMES', 'TMS')),
         Muscle = as.factor(ifelse(grepl('biceps', Muscle), 'Biceps\nbrachii', 
                                   'Triceps\nbrachii')))

# reorder factors for plotting
all_data$Task = factor(all_data$Task,
                       levels = c('Cycle', 'Tonic'))
all_data$Clock = factor(all_data$Clock, 
                        levels = c("6 o'clock",
                                   "12 o'clock"))
all_data$Stim = factor(all_data$Stim, 
                       levels = c('TMS', 'TMES'))
all_data$Muscle = factor(all_data$Muscle,
                         levels = c('Biceps\nbrachii',
                                    'Triceps\nbrachii'))


# Export number of trials ------------------------------------------------------
# for n numbers
trials_per_subject <- all_data %>% 
  group_by(Condition, Subject, Muscle)%>%
  summarise(N = n()) %>%
  ungroup() %>%
  pivot_wider(names_from = Subject, values_from = N) %>%
  group_by(Condition, Muscle) %>% 
  mutate(Total = sum(c_across(aa:rd)),
         Min = min(c_across(aa:rd)),
         Max = max(c_across(aa:rd)),
         across(where(is.numeric), replace_na, 0),
         Total_subjects = sum(c_across(aa:rd) != 0))%>% 
  arrange(Muscle, Condition)

# save as csv
# write.csv(trials_per_subject, paste(root_dir,'/N_trials_per_sub.csv', sep=''))


# Aggregate data ---------------------------------------------------------------
# calculate mean, mean of means, and sem for plot
Mean <- all_data %>%
  group_by(Subject, Condition, Task, Stim, Clock, Muscle) %>%
  summarise(P1 = round(mean(P1), digits = 2)) %>%
  arrange(Condition)

# calculate mean of mean and y positions for positive and negative error
Meanofmean <- Mean %>%
  group_by(Condition, Task, Stim, Clock, Muscle) %>%
  summarise(Latency = round(mean(P1), digits = 2),
            SE = round(sem(P1), digits = 2)) %>%
  mutate(error_neg = Latency-SE,
         error_pos = Latency+SE) %>%
  arrange(Condition)


# Sort data for bar charts of mean of means showing SEM  -----------------------

# for error bar widths
wth <- 0.6

# stats (One way unpaired t-test biceps vs triceps) and significance marking
stat.test <- all_data %>%
  group_by(Clock, Task, Stim) %>%
  tukey_hsd(P1 ~ Muscle + Subject, "Muscle") %>%
  adjust_pvalue(p.col = "p.adj", output.col = "p.adj", method = "bonferroni") %>%
  # add_significance(p.col = "p.adj", output.col = "p.adj.signif") %>%
  add_significance(p.col = "p.adj",
                   output.col = "p.adj.signif",
                   cutpoints = c(0, .01, 1),
                   # symbols = c("*\n|", "ns") # Add vertical line below *
                   symbols = c("*", "ns")) %>%
  add_xy_position(
    fun = "mean_se",
    comparisons = list(c("Biceps\nbrachii", "Triceps\nbrachii")),
    x = "Stim",
    dodge = wth,
    group = "Muscle",
    stack = FALSE,
    step.increase = 0)


# Plot bar charts of mean of means showing SEM  --------------------------------

# Set width of bars
err_bar_wth <- 0.3
bracket_thickess <- .5

# generate plot and aesthetics. Group required for signif
plot_out <- ggplot(Meanofmean, aes(x = Stim, y = Latency,
                                   group = Muscle)) +
  facet_grid(Clock ~ Task,
             switch = 'y') + 
  # Plot error bar first to get +ive error only
  geom_errorbar(aes(ymin=error_neg, ymax=error_pos),
                width = err_bar_wth,
                size = bracket_thickess,
                position = position_dodge(wth),
                stat = "identity") +
  # Plot significance.
  stat_pvalue_manual(stat.test,
                     label = "p.adj.signif",
                     hide.ns = TRUE,
                     bracket.size = bracket_thickess,
                     tip.length	= .05,
                     bracket.nudge.y = 1,
                     vjust = 0.4,
                     size = 8) +
  # Plot bars. Comes after signif, fill specified.
  geom_bar(data = Meanofmean, aes(x = Stim, y = Latency,
                                  fill = Muscle),
           width = wth,
           position = "dodge",
           stat ="identity") +
  # Plotting parameters
  scale_fill_manual(values = c("green", "blue")) +
  theme_classic2() +
  scale_y_continuous(limits = c(0, 30),
                     expand = c(0,0)) +
  # panel positions
  theme(panel.spacing = unit(.5, "cm", data = NULL)) +
  theme(strip.background = element_blank()) +
  theme(strip.placement.y = "outside") +
  # text
  # theme(text = element_text(size = 12)) +
  # theme(text = element_text(family = "Calibri")) +
  theme(legend.position="top") + 
  xlab('') +
  ylab(label = 'Time-to-peak (ms)')

# Visualize plot
plot(plot_out)

# parameters for saving
file_wdth <- 85 #190 for 2 column width
file_hght <- 100
file_name <- paste(ttp_fig_dir, "/Fig.3.2_TimeToPeak", sep="")

unlink(paste(file_name, ".tiff", sep=""))
unlink(paste(file_name, ".jpg", sep=""))
unlink(paste(file_name, ".pdf", sep=""))
unlink(paste(file_name, ".wmf", sep=""))
unlink(paste(file_name, ".eps", sep=""))
unlink(paste(file_name, ".emf", sep=""))


# ggplot2::ggsave(paste(file_name, ".tiff", sep=""), plot_out, width = file_wdth,
#                 height = file_hght, units = "mm", device = "tiff", scale = 1,
#                 dpi = 500)

ggplot2::ggsave(paste(file_name, ".jpg", sep=""), plot_out, width = file_wdth,
                height = file_hght, units = "mm", device = "jpeg", scale = 1,
                dpi = 500)

ggplot2::ggsave(paste(file_name, ".pdf", sep=""), plot_out, width = file_wdth,
                height = file_hght, units = "mm", device = "pdf", scale = 1,
                dpi = 500)

# ggplot2::ggsave(paste(file_name, ".wmf", sep=""), plot_out, width = file_wdth,
#                 height = file_hght, units = "mm", scale = 1,
#                 dpi = 500)
# 
# ggplot2::ggsave(paste(file_name, ".eps", sep=""), plot_out, width = file_wdth,
#                 height = file_hght, units = "mm", scale = 1,
#                 dpi = 500)
# 
# ggplot2::ggsave(paste(file_name, ".emf", sep=""), plot_out, width = file_wdth,
#                 height = file_hght, units = "mm", scale = 1,
#                 dpi = 500)


# Print stats ------------------------------------------------------------------

# collect stats
stat.test.conds <- left_join(stat.test, Meanofmean %>%
                               select(Condition, Task, Stim, Clock) %>%
                               unique(),
                             by = c("Task", "Stim", "Clock")) %>%
  mutate(p_sig = ifelse(p.adj < 0.01, "p < 0.01", "NS"))


# print test condition
print(paste("Two-way ANOVA, Subject + Muscle, p < 0.01, n = 10", sep=""))

# print test results for each comparison
for (condition in unique(Meanofmean$Condition)){
  cond_mean_sem <- Meanofmean %>%
    filter(Condition == condition)
  
  bic_mean <- cond_mean_sem$Latency[cond_mean_sem$Muscle == "Biceps\nbrachii"]
  bic_sem <- cond_mean_sem$SE[cond_mean_sem$Muscle == "Biceps\nbrachii"]
  bic_n <- trials_per_subject$Total_subjects[trials_per_subject$Condition == condition &
                                               trials_per_subject$Muscle == "Biceps\nbrachii"]
  tric_mean <- cond_mean_sem$Latency[cond_mean_sem$Muscle == "Triceps\nbrachii"]
  tric_sem <- cond_mean_sem$SE[cond_mean_sem$Muscle == "Triceps\nbrachii"]
  tric_n <- trials_per_subject$Total_subjects[trials_per_subject$Condition == condition &
                                                trials_per_subject$Muscle == "Triceps\nbrachii"]
  
  expression <- paste(condition, ": biceps brachii: ", bic_mean, "  ", bic_sem, " ms, n = ", bic_n,
                      "; triceps brachii: ", tric_mean, "  ", tric_sem, " ms, n = ", tric_n, sep="")
  
  # check the stats
  cond_stats <- stat.test.conds %>%
    filter(Condition == condition)
  
  # print(paste(expression, "; ", cond_stats$p_sig, sep="")) # for p < 0.01 
  print(paste(expression, "; ", format(round(cond_stats$p.adj, 3), nsmall = 3), sep="")) # for actual p val 
  
}


# Print difference in latency between MEPS and CMEPS ---------------------------
mep_cmep_dif <- Meanofmean %>%
  pivot_wider(id_cols = Muscle, names_from = Condition, 
              values_from = Latency) %>%
  group_by(Muscle) %>%
  summarise(cycle_6 = mep_cycle_6 - cmep_cycle_6,
            cycle_12 = mep_cycle_12 - cmep_cycle_12,
            iso_6 = mep_iso_6 - cmep_iso_6,
            iso_12 = mep_iso_12 - cmep_iso_12)

# write.csv(mep_cmep_dif, "Timetopeak_MEP-CMEP_dif.csv", quote = TRUE)


# Export mean time to peaks ----------------------------------------------------
Meanofmean_output <- Mean %>%
  mutate(Muscle = ifelse(grepl('Biceps\nbrachii', Muscle), 'biceps', 
                         'triceps')) %>%
  pivot_wider(id_cols = c(Subject, Condition), 
              names_from = Muscle, values_from = c(P1)) %>%
  group_by(Condition) %>%
  mutate(dif = biceps - triceps) %>%
  ungroup() %>%
  group_by(Condition) %>%
  summarise(Biceps = round(mean(biceps), digits = 2),
            Triceps = round(mean(triceps), digits = 2),
            Dif = round(mean(dif), digits = 2),
            Biceps_sem = round(sem(biceps), digits = 2),
            Triceps_sem = round(sem(triceps), digits = 2),
            Biceps_sd = round(sd(biceps), digits = 2),
            Triceps_sd = round(sd(triceps), digits = 2),
            Dif_sem = round(sem(dif), digits = 2),
            Dif_sd = round(sd(dif), digits = 2)) %>%
  ungroup() %>%
  mutate(Order = order(c("mep_cycle_6",
                         "cmep_cycle_6",
                         "mep_cycle_12",
                         "cmep_cycle_12",
                         "mep_iso_6",
                         "cmep_iso_6",
                         "mep_iso_12",
                         "cmep_iso_12"))) %>%
  arrange(Order) %>%
  select(-Order)

# write.csv(Meanofmean_output, "Timetopeak_means_dif.csv", quote = TRUE)